import { world, system } from "@minecraft/server";

/**
 * Gets the score recorded for {displayName} on {objective}
 * @param {String} target or entity on the scoreboard
 * @param {String} objectiveId Objective Identifer to get from
 * @param {Boolean} useZero If the return should be NaN if its not found or 0.
 * @returns {Number} Score that Was recorded for {Player} on {Objective}
 * @example getScore(player, "objective"): number
 */
export function getScore(target, objectiveId, useZero = true) {
    try {
        const oB = world.scoreboard.getObjective(objectiveId)
        if (typeof target != 'string') { target = target.name }
        return oB.getScore(oB.getParticipants().find(pT => pT.displayName == target))
    } catch {
        return useZero ? 0 : NaN
    }
}

export async function edScore(target, objective='data', value=0, operator='set') {
    if (typeof target == 'object') { target = target.name }
    if (!target.startsWith('@')) {target = `"${target}"`}
    await runCMD(`scoreboard players ${operator} ${target} ${objective} ${value}`)
}

/**
 * Fires 1 second delay,
 * @param {() => void} callback
 * @param {number} tick
 * @example setTickTimeout(() => { /* code *\/ }, 20)
*/
export function setTickTimeout(callback, tick) {
    const id = system.runInterval(() => {
      callback();
      system.clearRun(id);
    }, tick);
}

export function shuffle(array) {
    let currentIndex = array.length,  randomIndex;
    while (currentIndex != 0) {
        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex--;
        [array[currentIndex], array[randomIndex]] = [
            array[randomIndex], array[currentIndex]];
    }
    return array;
}

export function getItemAmounts(player, items) {
    const inv = player.getComponent('inventory')?.container;
    if (!inv) return;
    const counts = {};
    for (let id of items) {
      counts[id] = 0;
    }
    for (let i = 0; i < inv.size; i++) {
      const item = inv.getItem(i);
      if (!item) continue;
      const itemId = item.typeId.replace('minecraft:', '')
      const id = items.find(v => itemId == v.replace('minecraft:', ''))
      if (id) counts[id] += item.amount;
    }
    return counts;
  }

export function randomInt(min, max) { return Math.floor(Math.random() * (max - min + 1) + min) }

export async function playsound(sound,player='@a',volume=1,pitch=1) {
    await runCMD(`playsound ${sound} @s ~~~ ${volume} ${pitch}`,player)
}

export function axlog(msg) {
    console.warn(`[AxisLog] ${msg}`)
}

export async function rawtext(text,name='@a',type='text',color='r') {
    await runCMD(`tellraw @s {"rawtext":[{"text":"§${color}"},{"${type}":"${text}"}]}`,name)
}

export function actionbar(text,name='@a',color='§r') {
    runCMD(`title @s actionbar ${color}${text}`,name)
}

export async function tellraw(rawtext,name='@a') {
    await runCMD(`tellraw @s ${rawtext}`,name)
}

export function hasTag(source, tag){
    return source.getTags().indexOf(tag) != -1
  }

export async function runCMD(command, source, needLog = false){
    try {
        if (source == undefined) {
            await world.getDimension('overworld').runCommandAsync(command)
        } else if (typeof source == 'object') {
            await source.runCommandAsync(command)
            return
        } else if (typeof source == 'string' && source.startsWith('@')) {
            await world.getDimension('overworld').runCommandAsync(`execute as ${source} at @s run execute positioned as @s run ${command}`)
            return
        } else if (typeof source == 'string' && !source.startsWith('@')) {
            await world.getDimension('overworld').runCommandAsync(`execute as "${source}" at @s run execute positioned as @s run ${command}`)
            return
        }
    } catch(error) {
        if (needLog) {
            console.error(`Command: ${command}\nError: ${error}`)
        }
    }
};

export async function runCMDs(commands, source, needLog = false){
    for (let i in commands) {
        let thisCommand = commands[i]
        if (typeof thisCommand == 'string') {
            await runCMD(thisCommand,source,needLog)
        } else if (typeof thisCommand == 'object') {
            let target = thisCommand.target
            let target2 = target
            if (target == undefined) { target = '@a'; target2 = '@s' }
            if (target.startsWith('@s')) { target2 = target; target = source }
            switch (thisCommand.type) {
                case 'lockslot':
                    await runCMD(`replaceitem entity ${target2} slot.hotbar ${thisCommand.slot-1} ${thisCommand.item} 1 0 {"minecraft:item_lock":{ "mode": "lock_in_slot" }}`,target)
                break;
                case 'armor':
                    const elements = ['slot.armor.head','slot.armor.chest','slot.armor.legs','slot.armor.feet']
                    //replaceitem entity @s slot.armor.head
                    for (let i in thisCommand.elements) {
                        await runCMD(`replaceitem entity ${target2} ${elements[i-1]} 0 ${thisCommand.elements[i]} 1 0 {"minecraft:item_lock":{ "mode": "lock_in_inventory" }}`,target)
                    }
                break;
                case 'sound':
                    await runCMD(`playsound ${thisCommand.sound} ${target2} ~~~ ${thisCommand.v} ${thisCommand.p}`,target)
                break;
            }
        }
        
    }
};